@@ -82,6 +82,12 @@ typedef enum {
 	HTMLColors
 } HTMLColorId;
 
+typedef enum {
+	HTML_DIRECTION_DERIVED,
+	HTML_DIRECTION_LTR,
+	HTML_DIRECTION_RTL
+} HTMLDirection;
+
 typedef enum {
 	HTML_ENGINE_CURSOR_UP,
 	HTML_ENGINE_CURSOR_DOWN,
@@ -218,4 +224,48 @@ typedef enum {
 	HTML_VALIGN_NONE
 } HTMLVAlignType;
 
+typedef enum {
+	DISPLAY_NONE,
+	DISPLAY_INLINE,
+	DISPLAY_LIST_ITEM,
+	DISPLAY_INLINE_BLOCK,
+	DISPLAY_BLOCK,
+	DISPLAY_MARKER,
+	DISPLAY_RUN_IN,
+	DISPLAY_COMPACT,
+	DISPLAY_TABLE_ROW_GROUP,
+	DISPLAY_TABLE_COLUMN,
+	DISPLAY_TABLE_COLUMN_GROUP,
+	DISPLAY_TABLE_HEADER_GROUP,
+	DISPLAY_TABLE_FOOTER_GROUP,
+	DISPLAY_TABLE_ROW,
+	DISPLAY_TABLE_CELL,
+        DISPLAY_TABLE_CAPTION,
+	DISPLAY_INLINE_TABLE,
+	DISPLAY_TABLE,
+	DISPLAY_DOCUMENT
+} HTMLDisplayType;
+
+typedef enum {
+	HTML_WHITE_SPACE_NORMAL,
+	HTML_WHITE_SPACE_PRE,
+	HTML_WHITE_SPACE_NOWRAP,
+	HTML_WHITE_SPACE_PRE_WRAP,
+	HTML_WHITE_SPACE_PRE_LINE,
+	HTML_WHITE_SPACE_INHERIT
+} HTMLWhiteSpaceType;
+
+typedef enum {
+	HTML_LENGTH_TYPE_PIXELS,
+	HTML_LENGTH_TYPE_PERCENT,
+	HTML_LENGTH_TYPE_FRACTION
+} HTMLLengthType;
+
+typedef enum {
+	HTML_BORDER_NONE,
+	HTML_BORDER_SOLID,
+	HTML_BORDER_INSET,
+	HTML_BORDER_OUTSET
+} HTMLBorderStyle;
+
 #endif
